/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * fs/library.h -- File library support.
 * (c) 2003-2008 Murat Deligonul
 */
#ifndef __FS_LIBRARY_H
#define __FS_LIBRARY_H

#include <vector>
#include <cstring>
#include "util/hash.h"
#include "util/counted_object.h"
#include "fs/fwd.h"
#include "fs/entry_data.h"
#include "fs/exceptions.h"

namespace fs {

class resolved_path;

class file_library {
private:
	typedef util::hash_table<const char *, directory *, 
			util::default_hasher<const char *>,
		       	util::fixed_cstring_predicate<strcmp> > hash_table_t;

	hash_table_t directories;
	directory * root;
	credentials * root_creds;
	flib_key * root_key;

public:
	file_library(const char *);
	~file_library();
		
	flib_key * create_key(const credentials *, int *);	
	
	/** 'system' calls **/
	file_entry * open(const flib_key *, const char *, int, int *);	
	file_entry * import(const flib_key *, const char *);	
	int save(const flib_key *, file_entry *);
	int close(file_entry *);
	int stat(const flib_key *, const char *, entry_data&);
	
	int unlink(const flib_key *, const char *);	
	int access(const flib_key *, const char *, int);
		
	int chdir(flib_key *, const char *);
	int rmdir(const flib_key *, const char *);
	directory * mkdir(const flib_key *, const char *, mode_t, int *);
	directory * mkdirs(const flib_key *, const char *, mode_t, int *);

	int chmod(const flib_key *, const char *, mode_t);

	int ls(const flib_key *, const char *, std::vector<entry_data>&);

	/** error codes **/
	enum {
		ERR_BASE	  = -200,
		ERR_BADFILE 	  = -200,
		ERR_CREATION 	  = -201,
		ERR_PRIVATE_FILE  = -202,
		ERR_NOT_PERMITTED = -203,
		ERR_DIRECTORY     = -206,
		ERR_NOT_DIRECTORY = -207,
		ERR_BUSY 	  = -208,
		ERR_FAILURE       = -209,
		ERR_OPEN_FILES    = -210,
		ERR_RMDIR_FAILURE = -211,
		ERR_MKDIR_FAILURE = -212,
		ERR_SYMLINK       = -213,
		ERR_ACCESS        = -214,
		ERR_EXISTS	  = -215,
		ERR_DOESNT_EXIST  = -216,
		ERR_EXISTS_ON_DISK= -217,
		ERR_CONFIG	  = -218,
		ERR_NO_KEY        = -219,
		ERR_NO_FILELIB    = -220,
	        ERR_MAX		  = -220
	};

	static const char * strerror(int);

private:
	/** implementation **/
	friend class resolved_path;

	int resolve_path(const flib_key *, const char *, resolved_path *);
	int access(const flib_key *, file_entry *, int);
	int access_entry_only(const flib_key *, file_entry *, int);
	directory * load_directory(const flib_key *, directory *, const char *, int *);

	bool can_delete(const flib_key * key, const file_entry * entry) const;
	
	static int normalize_path(const flib_key * , const char *, char *);	

private:
	// non-copyable
	file_library& operator=(const file_library&);
	file_library(const file_library&);
};

class resolved_path {
private:
	char resolved[PATH_MAX+1];
	char * name_ptr;
	directory * _dir;
	file_entry * _entry;

public:
	char * buffer() { 
		return resolved; 
	}
	directory * dir() const { 
		return _dir; 
	}
	file_entry * cached_entry() const {
		return _entry;
	}

	const char * full_path() const { 
		return resolved; 
	}
	const char * name() const { 
		return name_ptr; 
	}

	resolved_path() : 
		name_ptr(NULL), _dir(NULL), _entry(NULL) {
		resolved[0] = 0;
		name_ptr = &resolved[0];
	}

	friend int file_library::resolve_path(const flib_key *, const char *, resolved_path *);

private:
	// non-copyable
	resolved_path(const resolved_path&);
	resolved_path& operator= (const resolved_path&);
};

} // namespace fs 
#endif
