/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __IRC_ADDRESS_H
#define __IRC_ADDRESS_H

#include "util/strings.h"

namespace irc {

/**
 * Represents the "nickname!ident@host" address of an IRC user.
 * The nickname and ident@host parts are stored separately, allowing
 * the nickname to be used as a hash key.  This is useful since initially
 * only the nickname is known; later the ident@host information can be added
 * without invalidating the nickname and the hash key.
 */
class address {
private:
	char * nickname;
	char * ident_host;

public:
	/**
	 * Construct address from a string containing a full address or 
	 * just a nickname. 
	 */
	explicit address(const char *);

	/*
	 * Construct address from 3 different parts.
	 */
	address(const char * n, const char * i, const char * h)
		: nickname(NULL), ident_host(NULL) {
		assert(n != NULL);
		set_nick(n);
		set_ident_host(i, h);
	}
	
	/**
	 * Construct address from another address.
	 */
	address(const address &that) : nickname(NULL), ident_host(NULL) {
		set_nick(that.nick());
		set_ident_host(that.ident(), that.host());
	}

	/**
	 * Destructor.
	 */
	~address() {
		delete[] nickname;
		delete[] ident_host;
	}

	/** 
	 * Assignment operator: primarily used by testing code.
	 */
	address& operator = (const address& that) {
		if (this == &that) {
			return *this;
		}
		set_nick(that.nick());
		set_ident_host(that.ident(), that.host());
		return *this;
	}

	const char * nick() const { 
		return nickname;
	}

	const char * ident() const {
		if (!has_ident_host()) {
			return "unknown";
		}
		return ident_host;
	}

	const char * host() const {
		if (!has_ident_host()) {
			return "unknown.host";
		}
		return ident_host+strlen(ident_host)+1;
	}

	bool has_ident_host() const {
		return ident_host != NULL;
	}

	void set_nick(const char * n) {
		assert(n != NULL);
		delete[] nickname;
		nickname = util::strings::my_strdup(n);
	}
	void set_ident_host(const char *, const char *);
};

} // namespace irc 
#endif
