/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * commands/irc.cc
 * (c) 2007-2008 Murat Deligonul
 */

#include "autoconf.h"

#include <vector>
#include <string>
#include <list>
#include <cstdlib>
#include "irc/address.h"
#include "irc/cache.h"
#include "logging/chatlog.h"
#include "conn.h"
#include "commands.h"
#include "textline.h"
#include "messages.h"
#include "debug.h"

/**
  * This file contains traps for outgoing standard IRC commands:
  * PRIVMSG, NOTICE, QUIT and others.
  */

using std::list;
using std::string;
using std::vector;
using irc::channel;

using namespace util::strings;

CMDFUNC(quit)
{
	if (decide<server_node>(servinfo->server_target(), 
			       	irc_server_config::AUTO_DETACH) == 1) {
		do_auto_detach();
		return 1;
	}
	return 0;
}

/**
 * Outgoing PRIVMSG handler -- Current does:
 *  Outgoing DCC proxying (and that's it) 
 */
CMDFUNC(privmsg)
{
	using logging::chatlog;
	const bool do_dcc_out = (decide<server_node>(get_server_target(), 
							irc_server_config::PROXY_DCC_OUT) > 0) ||
			  	  (decide<server_node>(get_server_target(),
						  	irc_server_config::FILTER_DCCS) > 0);
	const char * target = args[0];
	const char * token1 = no_leading(args[1]);

	if (is_bounced()) {
		const char * message = no_leading(args.get_rest(1));
		/* Log if Necessary */
		log_irc_event( servinfo->is_channel(target) ? chatlog::EVENT_PUBLIC : chatlog::EVENT_PRIVATE,
				irc, target, message, NULL);
	}
	if (do_dcc_out) {
		/** 
		 * If we get here, the message is in the form
		 * PRIVMSG target :[CTCP] xxxx
		 */
		if (args < 2 || token1[0] != '\001') {
			return 0;
		}
		const char * ctcp_args = args.get_rest(2);
		return handle_dcc(false, &token1[1], irc->nick(), args[0], ctcp_args);
	}

	return 0;
}

/**
  * Handle outgoing NOTICEs and log; currently does nothing.
  */
CMDFUNC(notice)
{
	return 0;
}

/** 
 * During reattach, capture who commands and queue them with the
 * the throttle timer. Most IRC clients will send 'WHO #chan' upon
 * joining, which is an expensive operation.
 */
CMDFUNC(who)
{
	if (args < 1) {		
		return 0;
	}
	channel * c = acache->lookup_channel(args[0]);
	if (!c) {
		return 0;
	}

	if (c->chan_flags() & channel::WHO_REQ_SENT) {
		DEBUG("conn::do_who_command(): Suppressing outgoing WHO command for %s\n", c->name());
		c->set_chan_flag(channel::RELAY_WHO_RESP);
		return 1;
	}
	if (c->chan_flags() & channel::WHO_DATA_RECVED) {
		// create the restartable procedure if needed
		if (recently_reattached()) {
			rfunc * rf = acache->create_rfunc(acache->RFUNC_WHO,
					client, c->name(), irc->nick());
			if (!rf) {
				return 0;
			}
			rfuncs.push_back(rf);
			DEBUG("conn::do_who_command() : created WHO rfunc for %s on %s\n", irc->nick(), c->name());
			return 1;
		}
	}
	return 0;
}

/**
  * Outgoing MODE commands.
  * Clients will send these after joining, and we do too.
  * 
  */
CMDFUNC(mode) 
{
	if (args < 1) {		
		return 0;
	}
	channel * c = acache->lookup_channel(args[0]);
	if (!c) {
		return 0;
	}

	if (c->chan_flags() & channel::MODE_REQ_SENT) {
		DEBUG("conn::do_who_command(): Suppressing outgoing MODE command for %s\n", c->name());
		c->set_chan_flag(channel::RELAY_MODE_RESP);
		return 1;
	}
	if (c->chan_flags() & channel::MODE_DATA_RECVED) {
		// XXX: do anything here?
		if (client) {
		} 
	}
	return 0;
}

/**
 * TODO: implement.
 */
CMDFUNC(protoctl) 
{
	return 0;
}

/**
 * TODO: implement.
 */
CMDFUNC(cap)
{
	return 0;
}
